<?php

	/**
	 * Main object browser.  This page first shows a list of databases and then
	 * if you click on a database it shows a list of database objects in that
	 * database.
	 *
	 * $Id: browser.php,v 1.13 2003/06/22 09:43:21 chriskl Exp $
	 */

	// Include application functions
	include_once('libraries/lib.inc.php');
	
	// Include tree classe
	include_once('classes/HTML_TreeMenu/TreeMenu.php');

	// Output header
	$misc->printHeader('', "<script src=\"classes/HTML_TreeMenu/TreeMenu.js\" type=\"text/javascript\"></script>");
	$misc->printBody('browser');
	
	// Construct expanding tree
	$menu  = new HTML_TreeMenu(null, array('usePersistence' => false));
	$root = new HTML_TreeNode(array(
						'text' => addslashes($misc->printVal(($conf['servers'][$_SESSION['webdbServerID']]['desc']))), 
						'link' => 'all_db.php', 
						'icon' => 'folder.gif', 
						'expandedIcon' => 'folder-expanded.gif',
						'expanded' => true,
						'linkTarget' => 'detail'));

	// Add root node to menu
	$menu->addItem($root);

	/**
	 * Helper function for adding nodes
	 * @param $schemanode Node onto which to add
	 */	
	function addNodes(&$schemanode, $querystr) {
		global $data, $localData, $misc, $lang, $conf;
		
		// Tables
		if ($data->hasTables()) {
			$table_node = &new HTML_TreeNode(array(
							'text' => addslashes($lang['strtables']), 
							'link' => addslashes(htmlspecialchars("tables.php?{$querystr}")), 
							'icon' => "../../../images/themes/{$conf['theme']}/tables.gif", 
							'expandedIcon' => "../../../images/themes/{$conf['theme']}/tables.gif",
							'expanded' => false,
							'linkTarget' => 'detail'));

			// Add table folder to schema
			$schemanode->addItem($table_node);
			
			$tables = &$localData->getTables();
			while (!$tables->EOF) {
				$item_node = &new HTML_TreeNode(array(
								'text' => addslashes($misc->printVal($tables->f[$data->tbFields['tbname']])), 
								'link' => addslashes(htmlspecialchars("tblproperties.php?{$querystr}&table=" .
									urlencode($tables->f[$data->tbFields['tbname']]))), 
								'icon' => "../../../images/themes/{$conf['theme']}/tables.gif", 
								'expandedIcon' => "../../../images/themes/{$conf['theme']}/tables.gif",
								'expanded' => false,
								'linkTarget' => 'detail'));

				// Add table folder to schema
				$table_node->addItem($item_node);

				$tables->moveNext();
			}
		}
		// Views
		if ($data->hasViews()) {
			$view_node = &new HTML_TreeNode(array(
							'text' => addslashes($lang['strviews']), 
							'link' => addslashes(htmlspecialchars("views.php?{$querystr}")), 
							'icon' => "../../../images/themes/{$conf['theme']}/views.gif", 
							'expandedIcon' => "../../../images/themes/{$conf['theme']}/views.gif",
							'expanded' => false,
							'linkTarget' => 'detail'));

			// Add folder to schema
			$schemanode->addItem($view_node);
		}
		// Sequences
		if ($data->hasSequences()) {
			$seq_node = &new HTML_TreeNode(array(
							'text' => addslashes($lang['strsequences']), 
							'link' => addslashes(htmlspecialchars("sequences.php?{$querystr}")), 
							'icon' => "../../../images/themes/{$conf['theme']}/sequences.gif", 
							'expandedIcon' => "../../../images/themes/{$conf['theme']}/sequences.gif",
							'expanded' => false,
							'linkTarget' => 'detail'));

			// Add folder to schema
			$schemanode->addItem($seq_node);
		}
		// Functions
		if ($data->hasFunctions()) {
			$func_node = &new HTML_TreeNode(array(
							'text' => addslashes($lang['strfunctions']), 
							'link' => addslashes(htmlspecialchars("functions.php?{$querystr}")), 
							'icon' => "../../../images/themes/{$conf['theme']}/functions.gif", 
							'expandedIcon' => "../../../images/themes/{$conf['theme']}/functions.gif",
							'expanded' => false,
							'linkTarget' => 'detail'));

			// Add folder to schema
			$schemanode->addItem($func_node);
		}
		// Types
		if ($data->hasTypes()) {
			$func_node = &new HTML_TreeNode(array(
							'text' => addslashes($lang['strtypes']), 
							'link' => addslashes(htmlspecialchars("types.php?{$querystr}")), 
							'icon' => "../../../images/themes/{$conf['theme']}/types.gif", 
							'expandedIcon' => "../../../images/themes/{$conf['theme']}/types.gif",
							'expanded' => false,
							'linkTarget' => 'detail'));

			// Add folder to schema
			$schemanode->addItem($func_node);
		}
	}	

	$databases = &$data->getDatabases();
	while (!$databases->EOF) {
		// If database is selected, show folder, otherwise show document
		if (isset($_REQUEST['database']) && $_REQUEST['database'] == $databases->f[$data->dbFields['dbname']]) {
			$db_node = &new HTML_TreeNode(array(
								'text' => addslashes($misc->printVal($databases->f[$data->dbFields['dbname']])), 
								'link' => addslashes(htmlspecialchars('database.php?database=' . urlencode($databases->f[$data->dbFields['dbname']]))), 
								'icon' => "../../../images/themes/{$conf['theme']}/database.gif", 
								'expandedIcon' => "../../../images/themes/{$conf['theme']}/database.gif",
								'expanded' => true,
								'linkTarget' => 'detail'));
		
			// If database supports schemas, add the extra level of hierarchy
			if ($data->hasSchemas()) {
				$schemas = &$localData->getSchemas();
				while (!$schemas->EOF) {
					$localData->setSchema($schemas->f[$data->nspFields['nspname']]);
					// Construct database & schema query string
					$querystr = 'database=' . urlencode($databases->f[$data->dbFields['dbname']]). '&schema=' .
							urlencode($schemas->f[$data->nspFields['nspname']]);
					$schemanode = &new HTML_TreeNode(array(
									'text' => addslashes($misc->printVal($schemas->f[$data->nspFields['nspname']])), 
									'link' => addslashes(htmlspecialchars("schema.php?{$querystr}")), 
									'icon' => 'folder.gif', 
									'expandedIcon' => 'folder-expanded.gif',
									// Auto-expand your personal schema, if it exists.  Also expand schema if there is
									// only one schema in the database.
									'expanded' => ($schemas->f[$data->nspFields['nspname']] == $_SESSION['webdbUsername']
															|| $schemas->recordCount() == 1),
									'linkTarget' => 'detail'));

					addNodes($schemanode, $querystr);

					// Add schema to database
					$db_node->addItem($schemanode);

					$schemas->moveNext();
				}
			}
			// Database doesn't support schemas
			else {
				// Construct database query string
				$querystr = 'database=' . urlencode($databases->f[$data->dbFields['dbname']]);
				
				addNodes($db_node, $querystr);
			}

			// Add node to menu
			$root->addItem($db_node);

		} else {
			$db_node = &new HTML_TreeNode(array(
								'text' => addslashes($misc->printVal($databases->f[$data->dbFields['dbname']])), 
								'link' => addslashes(htmlspecialchars("{$_SERVER['PHP_SELF']}?database=" . urlencode($databases->f[$data->dbFields['dbname']]))), 
								'icon' => "../../../images/themes/{$conf['theme']}/database.gif", 
								'expandedIcon' => "../../../images/themes/{$conf['theme']}/database.gif",
								'expanded' => false,
								'linkTarget' => '_self'));
		
			// Add node to menu
			$root->addItem($db_node);
		}
		
		$databases->moveNext();
	}		
	// Create the presentation class
	$treeMenu = &new HTML_TreeMenu_DHTML($menu, array('images' => 'classes/HTML_TreeMenu/images', 'defaultClass' => 'treeMenuDefault'));
	
	// Actually display the menu
	$treeMenu->printMenu();

   // Output footer
   $misc->printFooter();

?>
